VERSION = "API_V1"

def url_get_orders(*, url_base: str) -> str:
    return f"{url_base}/{VERSION}/ordenes"

def url_get_order(*, url_base: str, idOrden: str) -> str:
    return f"{url_base}/{VERSION}/ordenes/{idOrden}"


from requests import Response

from fenicio.orders import ParamsOrdersGET
from fenicio.request import get_orders, get_order

class Fenicio:
    """
    ### Fenicio Python SDK
    - https://developers.fenicio.help/es/apiv1/intro
    """
    def __init__(self, url_base: str):
        self.url_base = url_base.strip("/")
    
    def get_orders(self, *, params: ParamsOrdersGET = None) -> Response:
        return get_orders(url_base=self.url_base, params=params)

    def get_order(self, *, idOrden: str | int) -> Response:
        return get_order(url_base=self.url_base, idOrden=idOrden)



from typing import Literal, List
from datetime import datetime

from pydantic import BaseModel, Field

from fenicio._etc import BaseModelNoExtra


__all__ = ["FenicioOrders", "FenicioOrder"]

BuyerDocument = Literal["DOCUMENTO_IDENTIDAD", "PASAPORTE"]
Gender = Literal[None, "F", "M"]
OrderState = Literal[
    "EN_CURSO",
    "ABANDONADA",
    "PAGO_PENDIENTE",
    "REQUIERE_APROBACION",
    "APROBADA",
    "CANCELADA"
]
Country = Literal[
    "UY", "AR", "CL", "US", "AU", "ES", "CO",
    "PE", "BR", "BO", "PY", "CA", "RU", "MX",
    "NP", "UG", "UZ", "FR", "VE", "DE", "IT",
    "AT", "CN", "IL", "CR", "AS", "VN", "IN",
    "GB" # FIXME: No agregar manualmente, incluir todos los paises.
]
DeliveryState = Literal[
    None,
    "RECIBIDO",
    "PREPARANDO",
    "AGUARDANDO_DESPACHO",
    "EN_TRANSITO",
    "LISTO_PARA_RETIRAR",
    "ENTREGADO"
]



class ParamsOrdersGET(BaseModel):
    pag: int | None = Field(default=1, ge=1)
    tot: int = Field(default=50, ge=1, le=500)
    #fDesde: date = None
    estado: OrderState = None

    def to_json(self) -> dict:
        """ Retorna el JSON para usar en el requests."""
        return {k: v for k, v in self.model_dump().items() if v is not None}


class FenicioBuyerDocument(BaseModelNoExtra):
    numero: str
    pais: Country
    tipo: BuyerDocument

class FenicioBuyer(BaseModelNoExtra):
    id: int
    codigo: None
    email: str
    nombre: str | None
    apellido: str | None
    telefono: str | None
    genero: Gender
    documento: FenicioBuyerDocument | None
    extras: dict | None

class FenicioSchedule(BaseModelNoExtra):
    desde: datetime
    hasta: datetime

class FenicioDeliveryAddress(BaseModelNoExtra):
    latitud: float | None
    longitud: float | None
    pais: str
    estado: str
    localidad: str
    calle: str
    numeroPuerta: str
    numeroApto: str | None
    codigoPostal: str | None
    observaciones: str | None

class FenicioDeliveryService(BaseModelNoExtra):
    id: int
    codigo: str
    nombre: str

class FenicioDelivery(BaseModelNoExtra):
    tipo: Literal[None, "ENVIO", "RETIRO"]
    estado: DeliveryState
    horario: FenicioSchedule | None
    destinatario: str
    direccionEnvio: FenicioDeliveryAddress | None
    local: str | None
    servicioEntrega: FenicioDeliveryService | None
    codigoTracking: str | None
    etiqueta: str | None

class FenicioOrderProduct(BaseModelNoExtra):
    nombre: str
    sku: str
    cantidad: int
    cantidadRegalo: int
    codigoPrecio: str
    precio: float
    descuentos: list | None                 # TODO: Ver el modelo de datos.
    atributos: dict | None                  # TODO: Ver el modelo de datos.

class FenicioOrder(BaseModelNoExtra):
    """ https://developers.fenicio.help/es/referencias/modelos-de-datos?id=orden"""
    idOrden: str                            # String numérico.
    idOrdenOrigen: str
    numeroOrden: int                        # OBSOLETO: https://developers.fenicio.help/es/referencias/modelos-de-datos?id=orden
    referencia: str | None
    estado: OrderState
    motivoCancelacion: str | None
    origen: Literal["WEB", "CALLCENTER", "MERCADOLIBRE"]
    fechaInicio: datetime
    fechaAbandono: datetime | None
    fechaRecuperada: datetime | None
    fechaFin: datetime | None
    fechaCancelada: datetime | None
    comprador: FenicioBuyer | None
    direccionFacturacion: dict | None       # TODO: Ver su modelo de datos.
    codigoTributario: str | None
    razonSocial: str | None
    moneda: Literal["UYU"]
    pago: dict | None                       # TODO: Ver su modelo de datos.
    entrega: FenicioDelivery
    lineas: List[FenicioOrderProduct]
    impuestos: float
    importeTotal: int
    observaciones: str | None
    historialCallCenter: list

class FenicioOrders(BaseModelNoExtra):
    """ https://developers.fenicio.help/es/apiv1/ordenes-listar"""
    error: bool
    msj: str
    totAbs: int
    ordenes: List[FenicioOrder]



import requests
from requests import Response

from fenicio.url import url_get_orders, url_get_order
from fenicio.orders import ParamsOrdersGET

def get_orders(*, url_base: str, params: ParamsOrdersGET = None) -> Response:
    """ https://developers.fenicio.help/es/apiv1/ordenes-listar"""
    if params is None:
        params = ParamsOrdersGET()
    url = url_get_orders(url_base=url_base)
    return requests.get(url=url, params=params.to_json())

def get_order(*, url_base: str, idOrden: str | int) -> Response:
    """ https://developers.fenicio.help/es/apiv1/orden-recuperar"""
    url = url_get_order(url_base=url_base, idOrden=idOrden)
    return requests.get(url=url)



from pathlib import Path
from setuptools import find_packages, setup

setup(
    name = "fenicio",
    version = "0.0.2",
    author = "Alejo Prieto Dávalos",
    author_email = "alejoprietodavalos@gmail.com",
    packages = find_packages(),
    description = "Python SDK para la API de Fenicio https://fenicio.io/.",
    long_description = Path("README.md").read_text(),
    long_description_content_type = "text/markdown",
    url = "https://pypi.org/project/fenicio/",
    project_urls = {
        "Source": "https://github.com/AlejoPrietoDavalos/fenicio/"
    },
    python_requires = ">=3.12",
    install_requires = [
        "requests>=2.32.3",
        "pydantic>=2.8.2",
    ],
    include_package_data = True
)
